#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals


__license__   = 'GPL v3'
__copyright__ = u'2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'

load_translations()


import os, sys, platform
from os.path import expanduser
from calibre.utils.config import JSONConfig

class JsonConfigWrapper():
    '''
    A Wrapper of the JSONConfig class
    It can:
    - Iterate through the settings
    - Create a duplicate
    '''
    
    def __init__(self, path = None):
        '''
        Initialization
        '''
        if not path:
            self.prefs = {}                     # for duplicates we use a dictionary
            return
        
        self.path = path
        self.prefs = JSONConfig(path)

        self.prefs.defaults['max_level'] = 2
        self.prefs.defaults['use_strip'] = False
        self.prefs.defaults['strip'] = '/^ *\d*\.?\d* */'
        self.prefs.defaults['omit'] = False
        self.prefs.defaults['skip'] = False
        self.prefs.defaults['continuous'] = False
        self.prefs.defaults['log_path'] = self.get_path('log', 'calibre.log')
        self.prefs.defaults['pdftk'] = 'C:/Programme (extra)/PDFtk Server/bin/pdftk.exe'
        self.prefs.defaults['ruby'] = 'Ruby.exe'
        
    
    def __getitem__(self, key):
        '''
        Access to the embedded configuration
        '''
        return self.prefs[key]
    
    
    def __setitem__(self, key, value):
        '''
        Access to the embedded configuration
        '''
        self.prefs[key] = value
        
        
    def __iter__(self):
        '''
        Iterates through the settings
        '''
        keys = ['max_level', 'use_strip', 'strip', 'omit', 'skip', 'continuous', 'log_path', 'pdftk', 'ruby']
        for key in keys:
            yield key, self[key]
        
        
    def dup(self):
        '''
        Provides a duplicate
        '''
        duplicate = JsonConfigWrapper()
        for key, value in self:
            duplicate[key] = value 
         
        return duplicate
    
    
    def get_path(self, relative = None, file = None):
        '''
        Determines an os dependent special directory
        - windows: documents
        - linux: home/documents
        and joins it with relative folder, creating the folder, if it does not exist
        file name is also appended, if present
        '''
        if platform.system() == 'Windows':
            import ctypes.wintypes
            CSIDL_PERSONAL = 5                          # My Documents
            SHGFP_TYPE_CURRENT = 0                      # Get current, not default value
            
            buf= ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
            ctypes.windll.shell32.SHGetFolderPathW(None, CSIDL_PERSONAL, None, SHGFP_TYPE_CURRENT, buf)
            
            path = buf.value
        else:
            path = expanduser(_('~/Documents'))
            
        if relative:
            path = os.path.join(path, relative)
            if not os.path.exists(path):
                os.makedirs(path)
            
        if file:
            path = os.path.join(path, file)
            
        return path
            
